<?php

namespace Drupal\facets_autocomplete\Plugin\facets\widget;

use Drupal\facets\FacetInterface;
use Drupal\facets\Result\ResultInterface;
use Drupal\facets\Widget\WidgetPluginBase;

/**
 * A widget that provides a textfield that autocomplets the facet results.
 *
 * @FacetsWidget(
 *   id = "autocomplete",
 *   label = @Translation("Textfield with autocomplete"),
 *   description= @Translation("A widget that provides an autocomplete."),
 * )
 */
class AutoCompleteWidget extends WidgetPluginBase {

  /**
   * {@inheritdoc}
   */
  public function build(FacetInterface $facet) {
//    $build = parent::build($facet);
//    $build['#attributes']['class'][] = 'js-facets-dropdown-links';
//    $build['#attached']['library'][] = 'facets/drupal.facets.dropdown-widget';

//    $build = [];
    /** @var \Drupal\facets\Result\Result[] $results */
    $results = $facet->getResults();
    $items = $autocomplete_values = $automplete_urls = [];

    $configuration = $facet->getWidget();
    $this->showNumbers = empty($configuration['show_numbers']) ? FALSE : (bool) $configuration['show_numbers'];

    foreach ($results as $result) {
      if (is_null($result->getUrl())) {
        // @TODO: figure out what this is and fill in.
        $text = $this->generateValues($result);
        $items[$facet->getFieldIdentifier()][] = $text;
      }
      else {
//        $items[$facet->getFieldIdentifier()][] = $this->buildListItems($facet, $result);
        $autocomplete_values[$result->getRawValue()] = $result->getDisplayValue() . ' (' . $result->getCount() . ')';
        $url = $result->getUrl()->toString();
        $automplete_urls[$result->getDisplayValue() . ' (' . $result->getCount() . ')'] = $url;
      }
    }

    $active = $facet->getActiveItems();
    $default_value = '';
    if (isset($active[0])) {
      $default_value = $autocomplete_values[$active[0]];
    }
    // Add librarys and everything to js.
    $build['#attached']['library'][] = 'facets_autocomplete/drupal.facets_autocomplete.autocomplete-widget';
    $build['#attached']['drupalSettings']['facets_autocomplete']['autocomplete_widget'][$facet->id()]['results'] = $autocomplete_values;
    $build['#attached']['drupalSettings']['facets_autocomplete']['autocomplete_widget'][$facet->id()]['urls'] = $automplete_urls;
    $build['#attached']['drupalSettings']['facets_autocomplete']['autocomplete_widget'][$facet->id()]['default_value'] = $default_value;

    // @TODO: add default value here instead of in js.
    $build['autocomplete_' . $facet->id()] = [
      '#type' => 'textfield',
      '#title' => '@TODO: title autocomplete',
      '#attributes' => [
        'class' => ['autocomplete-facet'],
        'data-id' => [$facet->id()],
        'id' => [$facet->id()],
      ],
      '#default_value' => $default_value,
    ];

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function buildListItems(FacetInterface $facet, ResultInterface $result) {
    if ($children = $result->getChildren()) {
      $items = $this->prepare($result);

      $children_markup = [];
      foreach ($children as $child) {
        $children_markup[] = $this->buildChildren($child);
      }

      $items['children'] = [$children_markup];

    }
    else {
      $items = $this->prepare($result);
    }

    return $items;
  }

  /**
   * Prepares the URL and values for the facet.
   *
   * @param \Drupal\facets\Result\ResultInterface $result
   *   A result item.
   *
   * @return array
   *   The results.
   */
  protected function prepare(ResultInterface $result) {
    $values = $this->generateValues($result);

    if (is_null($result->getUrl())) {
      $facet_values = $values;
    }
    else {
      $facet_values['url'] = $result->getUrl()->setAbsolute()->toString();
      $facet_values['values'] = $values;
    }

    return $facet_values;
  }

  /**
   * Builds an array for children results.
   *
   * @param \Drupal\facets\Result\ResultInterface $child
   *   A result item.
   *
   * @return array
   *   An array with the results.
   */
  protected function buildChildren(ResultInterface $child) {
    $values = $this->generateValues($child);

    if (!is_null($child->getUrl())) {
      $facet_values['url'] = $child->getUrl()->setAbsolute()->toString();
      $facet_values['values'] = $values;
    }
    else {
      $facet_values = $values;
    }

    return $facet_values;
  }

  /**
   * Generates the value and the url.
   *
   * @param \Drupal\facets\Result\ResultInterface $result
   *   The result to extract the values.
   *
   * @return array
   *   The values.
   */
  protected function generateValues(ResultInterface $result) {
    $values['value'] = $result->getDisplayValue();

    if ($this->getConfiguration()['show_numbers'] && $result->getCount() !== FALSE) {
      $values['count'] = $result->getCount();
    }

    if ($result->isActive()) {
      $values['active'] = 'true';
    }

    return $values;
  }

}
